@@ -6,22 +6,9 @@ import dataclasses
 import time
 import weakref
 from collections import defaultdict
+from collections.abc import Iterable, Mapping
 from contextvars import ContextVar
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Callable,
-    DefaultDict,
-    Iterable,
-    List,
-    Mapping,
-    Optional,
-    Set,
-    Tuple,
-    Type,
-    TypeVar,
-    Union,
-)
+from typing import TYPE_CHECKING, Any, Callable, Literal, TypeVar
 
 from typing_extensions import dataclass_transform
 
@@ -33,29 +20,29 @@ if TYPE_CHECKING:
 
 MAX_PROPAGATION_TIME = 0.01
 
-propagation_visited: ContextVar[Optional[Set[Tuple[int, str]]]] = ContextVar('propagation_visited', default=None)
+propagation_visited: ContextVar[set[tuple[int, str]] | None] = ContextVar('propagation_visited', default=None)
 
-bindings: DefaultDict[Tuple[int, str], List[Tuple[Any, Any, str, Optional[Callable[[Any], Any]]]]] = defaultdict(list)
-bindable_properties: weakref.WeakValueDictionary[Tuple[int, str], Any] = weakref.WeakValueDictionary()
-active_links: List[Tuple[Any, str, Any, str, Optional[Callable[[Any], Any]]]] = []
+bindings: defaultdict[tuple[int, str], list[tuple[Any, Any, str, Callable[[Any], Any] | None]]] = defaultdict(list)
+bindable_properties: weakref.WeakValueDictionary[tuple[int, str], Any] = weakref.WeakValueDictionary()
+active_links: list[tuple[Any, str, Any, str, Callable[[Any], Any] | None]] = []
 
 TC = TypeVar('TC', bound=type)
 T = TypeVar('T')
 
 
-def _has_attribute(obj: Union[object, Mapping], name: str) -> Any:
+def _has_attribute(obj: object | Mapping, name: str) -> Any:
     if isinstance(obj, Mapping):
         return name in obj
     return hasattr(obj, name)
 
 
-def _get_attribute(obj: Union[object, Mapping], name: str) -> Any:
+def _get_attribute(obj: object | Mapping, name: str) -> Any:
     if isinstance(obj, Mapping):
         return obj[name]
     return getattr(obj, name)
 
 
-def _set_attribute(obj: Union[object, Mapping], name: str, value: Any) -> None:
+def _set_attribute(obj: object | Mapping, name: str, value: Any) -> None:
     if isinstance(obj, dict):
         obj[name] = value
     else:
@@ -118,8 +105,30 @@ def _propagate_recursively(source_obj: Any, source_name: str) -> None:
             _propagate_recursively(target_obj, target_name)
 
 
+def _check_attribute_exists(other_obj: Any, other_name: str, *, role: Literal['self', 'other']) -> None:
+    if not _has_attribute(other_obj, other_name):
+        if isinstance(other_obj, Mapping):
+            raise KeyError(
+                f'Could not bind non-existing key "{other_name}". '
+                f'To allow missing keys (lazy binding), remove {role}_strict=True or add the key before binding.'
+            )
+        raise AttributeError(
+            f'Could not bind non-existing attribute "{other_name}" on object of type {other_obj.__class__.__name__}. '
+            f'To allow missing attributes (lazy binding), add {role}_strict=False or add the attribute before binding.'
+        )
+
+
+def _check_self_and_other_attribute(self_obj: Any, self_name: str, other_obj: Any, other_name: str,
+                                    self_strict: bool | None, other_strict: bool | None) -> None:
+    if self_strict or (self_strict is None and not isinstance(self_obj, dict)):
+        _check_attribute_exists(self_obj, self_name, role='self')
+    if other_strict or (other_strict is None and not isinstance(other_obj, dict)):
+        _check_attribute_exists(other_obj, other_name, role='other')
+
+
 def bind_to(self_obj: Any, self_name: str, other_obj: Any, other_name: str,
-            forward: Optional[Callable[[Any], Any]] = None) -> None:
+            forward: Callable[[Any], Any] | None = None, *,
+            self_strict: bool | None = None, other_strict: bool | None = None) -> None:
     """Bind the property of one object to the property of another object.
 
     The binding works one way only, from the first object to the second.
@@ -130,7 +139,12 @@ def bind_to(self_obj: Any, self_name: str, other_obj: Any, other_name: str,
     :param other_obj: The object to bind to.
     :param other_name: The name of the property to bind to.
     :param forward: A function to apply to the value before applying it (default: identity).
+    :param self_strict: Whether to check (and raise) if the first object has the specified property
+        (default: None, performs a check if the object is not a dictionary, *added in version 3.0.0*).
+    :param other_strict: Whether to check (and raise) if the second object has the specified property
+        (default: None, performs a check if the object is not a dictionary, *added in version 3.0.0*).
     """
+    _check_self_and_other_attribute(self_obj, self_name, other_obj, other_name, self_strict, other_strict)
     bindings[(id(self_obj), self_name)].append((self_obj, other_obj, other_name, forward))
     if (id(self_obj), self_name) not in bindable_properties:
         active_links.append((self_obj, self_name, other_obj, other_name, forward))
@@ -138,7 +152,8 @@ def bind_to(self_obj: Any, self_name: str, other_obj: Any, other_name: str,
 
 
 def bind_from(self_obj: Any, self_name: str, other_obj: Any, other_name: str,
-              backward: Optional[Callable[[Any], Any]] = None) -> None:
+              backward: Callable[[Any], Any] | None = None, *,
+              self_strict: bool | None = None, other_strict: bool | None = None) -> None:
     """Bind the property of one object from the property of another object.
 
     The binding works one way only, from the second object to the first.
@@ -149,7 +164,12 @@ def bind_from(self_obj: Any, self_name: str, other_obj: Any, other_name: str,
     :param other_obj: The object to bind from.
     :param other_name: The name of the property to bind from.
     :param backward: A function to apply to the value before applying it (default: identity).
+    :param self_strict: Whether to check (and raise) if the first object has the specified property (default: None,
+        performs a check if the object is not a dictionary, *added in version 3.0.0*).
+    :param other_strict: Whether to check (and raise) if the second object has the specified property (default: None,
+        performs a check if the object is not a dictionary, *added in version 3.0.0*).
     """
+    _check_self_and_other_attribute(self_obj, self_name, other_obj, other_name, self_strict, other_strict)
     bindings[(id(other_obj), other_name)].append((other_obj, self_obj, self_name, backward))
     if (id(other_obj), other_name) not in bindable_properties:
         active_links.append((other_obj, other_name, self_obj, self_name, backward))
@@ -157,8 +177,10 @@ def bind_from(self_obj: Any, self_name: str, other_obj: Any, other_name: str,
 
 
 def bind(self_obj: Any, self_name: str, other_obj: Any, other_name: str, *,
-         forward: Optional[Callable[[Any], Any]] = None,
-         backward: Optional[Callable[[Any], Any]] = None) -> None:
+         forward: Callable[[Any], Any] | None = None,
+         backward: Callable[[Any], Any] | None = None,
+         self_strict: bool | None = None,
+         other_strict: bool | None = None) -> None:
     """Bind the property of one object to the property of another object.
 
     The binding works both ways, from the first object to the second and from the second to the first.
@@ -171,14 +193,19 @@ def bind(self_obj: Any, self_name: str, other_obj: Any, other_name: str, *,
     :param other_name: The name of the second property to bind.
     :param forward: A function to apply to the value before applying it to the second object (default: identity).
     :param backward: A function to apply to the value before applying it to the first object (default: identity).
+    :param self_strict: Whether to check (and raise) if the first object has the specified property (default: None,
+        performs a check if the object is not a dictionary, *added in version 3.0.0*).
+    :param other_strict: Whether to check (and raise) if the second object has the specified property (default: None,
+        performs a check if the object is not a dictionary, *added in version 3.0.0*).
     """
-    bind_from(self_obj, self_name, other_obj, other_name, backward=backward)
-    bind_to(self_obj, self_name, other_obj, other_name, forward=forward)
+    _check_self_and_other_attribute(self_obj, self_name, other_obj, other_name, self_strict, other_strict)
+    bind_from(self_obj, self_name, other_obj, other_name, backward=backward, self_strict=False, other_strict=False)
+    bind_to(self_obj, self_name, other_obj, other_name, forward=forward, self_strict=False, other_strict=False)
 
 
 class BindableProperty:
 
-    def __init__(self, on_change: Optional[Callable[..., Any]] = None) -> None:
+    def __init__(self, on_change: Callable[..., Any] | None = None) -> None:
         self._change_handler = on_change
 
     def __set_name__(self, _, name: str) -> None:
@@ -237,9 +264,9 @@ def reset() -> None:
 
 
 @dataclass_transform()
-def bindable_dataclass(cls: Optional[TC] = None, /, *,
-                       bindable_fields: Optional[Iterable[str]] = None,
-                       **kwargs: Any) -> Union[Type[DataclassInstance], IdentityFunction]:
+def bindable_dataclass(cls: TC | None = None, /, *,
+                       bindable_fields: Iterable[str] | None = None,
+                       **kwargs: Any) -> type[DataclassInstance] | IdentityFunction:
     """A decorator that transforms a class into a dataclass with bindable fields.
 
     This decorator extends the functionality of ``dataclasses.dataclass`` by making specified fields bindable.
@@ -264,8 +291,8 @@ def bindable_dataclass(cls: Optional[TC] = None, /, *,
         if kwargs.get(unsupported_option):
             raise ValueError(f'`{unsupported_option}=True` is not supported with bindable_dataclass')
 
-    dataclass: Type[DataclassInstance] = dataclasses.dataclass(**kwargs)(cls)
-    field_names = set(field.name for field in dataclasses.fields(dataclass))
+    dataclass: type[DataclassInstance] = dataclasses.dataclass(**kwargs)(cls)
+    field_names = {field.name for field in dataclasses.fields(dataclass)}
     if bindable_fields is None:
         bindable_fields = field_names
     for field_name in bindable_fields:
@@ -277,12 +304,12 @@ def bindable_dataclass(cls: Optional[TC] = None, /, *,
     return dataclass
 
 
-def _make_copyable(cls: Type[T]) -> None:
+def _make_copyable(cls: type[T]) -> None:
     """Tell the copy module to update the ``bindable_properties`` dictionary when an object is copied."""
     if cls in copyreg.dispatch_table:
         return
 
-    def _pickle_function(obj: T) -> Tuple[Callable[..., T], Tuple[Any, ...]]:
+    def _pickle_function(obj: T) -> tuple[Callable[..., T], tuple[Any, ...]]:
         reduced = obj.__reduce__()
         assert isinstance(reduced, tuple)
         creator = reduced[0]
