@@ -5,9 +5,9 @@ import inspect
 import time
 import uuid
 from collections import defaultdict
-from contextlib import contextmanager
+from collections.abc import Awaitable, Iterable
 from pathlib import Path
-from typing import TYPE_CHECKING, Any, Awaitable, Callable, ClassVar, Dict, Iterable, Iterator, List, Optional, Union
+from typing import TYPE_CHECKING, Any, Callable, ClassVar
 
 from fastapi import Request
 from fastapi.responses import Response
@@ -32,42 +32,53 @@ if TYPE_CHECKING:
 
 templates = Jinja2Templates(Path(__file__).parent / 'templates')
 
+HTML_ESCAPE_TABLE = str.maketrans({
+    '&': '&amp;',
+    '<': '&lt;',
+    '>': '&gt;',
+    '`': '&#96;',
+    '$': '&#36;',
+})
+
+
+class ClientConnectionTimeout(TimeoutError):
+    def __init__(self, client: Client) -> None:
+        super().__init__(f'ClientConnectionTimeout: {client.id}')
+        self.client = client
+
 
 class Client:
-    page_routes: ClassVar[Dict[Callable[..., Any], str]] = {}
+    page_routes: ClassVar[dict[Callable[..., Any], str]] = {}
     '''Maps page builders to their routes.'''
 
-    instances: ClassVar[Dict[str, Client]] = {}
+    instances: ClassVar[dict[str, Client]] = {}
     '''Maps client IDs to clients.'''
 
-    auto_index_client: Client
-    '''The client that is used to render the auto-index page.'''
-
     shared_head_html = ''
     '''HTML to be inserted in the <head> of every page template.'''
 
     shared_body_html = ''
     '''HTML to be inserted in the <body> of every page template.'''
 
-    def __init__(self, page: page, *, request: Optional[Request]) -> None:
-        self.request: Optional[Request] = request
+    def __init__(self, page: page, *, request: Request | None = None) -> None:
+        self._request = request
         self.id = str(uuid.uuid4())
         self.created = time.time()
         self.instances[self.id] = self
 
-        self.elements: Dict[int, Element] = {}
+        self.elements: dict[int, Element] = {}
         self.next_element_id: int = 0
-        self._waiting_for_connection: asyncio.Event = asyncio.Event()
-        self.is_waiting_for_connection: bool = False
-        self.is_waiting_for_disconnect: bool = False
-        self.environ: Optional[Dict[str, Any]] = None
-        self.shared = request is None
+        self._waiting_for_connection = asyncio.Event()
+        self._waiting_for_disconnect = asyncio.Event()
+        self._connected = asyncio.Event()
+        self._deleted_event = asyncio.Event()
+        self.environ: dict[str, Any] | None = None
         self.on_air = False
         self._num_connections: defaultdict[str, int] = defaultdict(int)
-        self._delete_tasks: Dict[str, asyncio.Task] = {}
+        self._delete_tasks: dict[str, asyncio.Task] = {}
         self._deleted = False
-        self._socket_to_document_id: Dict[str, str] = {}
-        self.tab_id: Optional[str] = None
+        self._socket_to_document_id: dict[str, str] = {}
+        self.tab_id: str | None = None
 
         self.page = page
         self.outbox = Outbox(self)
@@ -77,48 +88,51 @@ class Client:
                 with Element('q-page'):
                     self.content = Element('div').classes('nicegui-content')
 
-        self.title: Optional[str] = None
+        self.title: str | None = None
 
         self._head_html = ''
         self._body_html = ''
 
         self.storage = ObservableDict()
 
-        self.connect_handlers: List[Union[Callable[..., Any], Awaitable]] = []
-        self.disconnect_handlers: List[Union[Callable[..., Any], Awaitable]] = []
+        self.connect_handlers: list[Callable[..., Any] | Awaitable] = []
+        self.disconnect_handlers: list[Callable[..., Any] | Awaitable] = []
+        self.delete_handlers: list[Callable[..., Any] | Awaitable] = []
 
-        self._temporary_socket_id: Optional[str] = None
+        self._temporary_socket_id: str | None = None
 
         with self:
             self.sub_pages_router = SubPagesRouter(request)
 
     @property
-    def is_auto_index_client(self) -> bool:
-        """Return True if this client is the auto-index client."""
-        return self is self.auto_index_client
+    def request(self) -> Request:
+        """The request object for the client."""
+        if self._request is None:
+            raise RuntimeError('Request is not set')
+        return self._request
 
     @property
-    def ip(self) -> Optional[str]:
-        """Return the IP address of the client, or None if it is an
-        `auto-index page <https://nicegui.io/documentation/section_pages_routing#auto-index_page>`_.
+    def ip(self) -> str:
+        """The IP address of the client.
 
         *Updated in version 2.0.0: The IP address is available even before the client connects.*
+        *Updated in version 3.0.0: The IP address is always defined (never ``None``).*
         """
-        return self.request.client.host if self.request is not None and self.request.client is not None else None
+        return self.request.client.host if self.request.client is not None else ''
 
     @property
     def has_socket_connection(self) -> bool:
-        """Return True if the client is connected, False otherwise."""
+        """Whether the client is connected."""
         return self.tab_id is not None
 
     @property
     def head_html(self) -> str:
-        """Return the HTML code to be inserted in the <head> of the page template."""
+        """The HTML code to be inserted in the <head> of the page template."""
         return self.shared_head_html + self._head_html
 
     @property
     def body_html(self) -> str:
-        """Return the HTML code to be inserted in the <body> of the page template."""
+        """The HTML code to be inserted in the <body> of the page template."""
         return self.shared_body_html + self._body_html
 
     def __enter__(self) -> Self:
@@ -131,7 +145,7 @@ class Client:
     def build_response(self, request: Request, status_code: int = 200) -> Response:
         """Build a FastAPI response for the client."""
         self.outbox.updates.clear()
-        prefix = request.headers.get('X-Forwarded-Prefix', request.scope.get('root_path', ''))
+        prefix = request.headers.get('X-Forwarded-Prefix', '') + request.scope.get('root_path', '')
         elements = json.dumps({
             id: element._to_dict() for id, element in self.elements.items()  # pylint: disable=protected-access
         })
@@ -148,11 +162,7 @@ class Client:
             context={
                 'request': request,
                 'version': __version__,
-                'elements': elements.replace('&', '&amp;')
-                                    .replace('<', '&lt;')
-                                    .replace('>', '&gt;')
-                                    .replace('`', '&#96;')
-                                    .replace('$', '&#36;'),
+                'elements': elements.translate(HTML_ESCAPE_TABLE),
                 'head_html': self.head_html,
                 'body_html': '<style>' + '\n'.join(vue_styles) + '</style>\n' + self.body_html + '\n' + '\n'.join(vue_html),
                 'vue_scripts': '\n'.join(vue_scripts),
@@ -182,37 +192,43 @@ class Client:
         """Return the title of the page."""
         return self.page.resolve_title() if self.title is None else self.title
 
-    async def connected(self, timeout: float = 3.0, check_interval: float = 0.1) -> None:
-        """Block execution until the client is connected."""
-        self.is_waiting_for_connection = True
-        self._waiting_for_connection.set()
-        deadline = time.time() + timeout
-        while not self.has_socket_connection:
-            if time.time() > deadline:
-                raise TimeoutError(f'No connection after {timeout} seconds')
-            await asyncio.sleep(check_interval)
-        self.is_waiting_for_connection = False
+    async def connected(self, timeout: float | None = None) -> None:
+        """Block execution until the client is connected.
 
-    async def disconnected(self, check_interval: float = 0.1) -> None:
+        :param timeout: timeout in seconds (default: ``None``)
+        """
+        if self.has_socket_connection:
+            return
+        self._waiting_for_connection.set()
+        self._connected.clear()
+        purpose = (self.request.headers.get('Sec-Purpose') or self.request.headers.get('Purpose') or '').lower()
+        is_prefetch = 'prefetch' in purpose and 'prerender' not in purpose
+        try:
+            await asyncio.wait_for(self._connected.wait(), timeout=None if is_prefetch else timeout)
+        except asyncio.TimeoutError as e:
+            raise ClientConnectionTimeout(self) from e
+
+    async def disconnected(self) -> None:
         """Block execution until the client disconnects."""
         if not self.has_socket_connection:
             await self.connected()
-        self.is_waiting_for_disconnect = True
-        while self.id in self.instances:
-            await asyncio.sleep(check_interval)
-        self.is_waiting_for_disconnect = False
+        if self.id in self.instances:
+            self._waiting_for_disconnect.set()
+            self._deleted_event.clear()
+            await self._deleted_event.wait()
 
     def run_javascript(self, code: str, *, timeout: float = 1.0) -> AwaitableResponse:
         """Execute JavaScript on the client.
 
-        The client connection must be established before this method is called.
-        You can do this by `await client.connected()` or register a callback with `client.on_connect(...)`.
-
         If the function is awaited, the result of the JavaScript code is returned.
         Otherwise, the JavaScript code is executed without waiting for a response.
 
+        Obviously the JavaScript code is only executed after the client is connected.
+        Internally, ``await client.connected()`` is called before the JavaScript code is executed (*since version 3.0.0*).
+        This might delay the execution of the JavaScript code and is not covered by the ``timeout`` parameter.
+
         :param code: JavaScript code to run
-        :param timeout: timeout in seconds (default: `1.0`)
+        :param timeout: timeout in seconds (default: 1.0)
 
         :return: AwaitableResponse that can be awaited to get the result of the JavaScript code
         """
@@ -223,33 +239,50 @@ class Client:
             self.outbox.enqueue_message('run_javascript', {'code': code}, target_id)
 
         async def send_and_wait():
-            if self is self.auto_index_client:
-                raise RuntimeError('Cannot await JavaScript responses on the auto-index page. '
-                                   'There could be multiple clients connected and it is not clear which one to wait for.')
             self.outbox.enqueue_message('run_javascript', {'code': code, 'request_id': request_id}, target_id)
+            await self.connected()
             return await JavaScriptRequest(request_id, timeout=timeout)
 
         return AwaitableResponse(send_and_forget, send_and_wait)
 
-    def open(self, target: Union[Callable[..., Any], str], new_tab: bool = False) -> None:
+    def open(self, target: Callable[..., Any] | str, new_tab: bool = False) -> None:
         """Open a new page in the client."""
         path = target if isinstance(target, str) else self.page_routes[target]
         self.outbox.enqueue_message('open', {'path': path, 'new_tab': new_tab}, self.id)
 
-    def download(self, src: Union[str, bytes], filename: Optional[str] = None, media_type: str = '') -> None:
+    def download(self, src: str | bytes, filename: str | None = None, media_type: str = '') -> None:
         """Download a file from a given URL or raw bytes."""
         self.outbox.enqueue_message('download', {'src': src, 'filename': filename, 'media_type': media_type}, self.id)
 
-    def on_connect(self, handler: Union[Callable[..., Any], Awaitable]) -> None:
-        """Add a callback to be invoked when the client connects."""
+    def on_connect(self, handler: Callable[..., Any] | Awaitable) -> None:
+        """Add a callback to be invoked when the client connects.
+
+        The callback has an optional parameter of `nicegui.Client`.
+        """
         self.connect_handlers.append(handler)
 
-    def on_disconnect(self, handler: Union[Callable[..., Any], Awaitable]) -> None:
-        """Add a callback to be invoked when the client disconnects."""
+    def on_disconnect(self, handler: Callable[..., Any] | Awaitable) -> None:
+        """Add a callback to be invoked when the client disconnects.
+
+        The callback has an optional parameter of `nicegui.Client`.
+
+        *Updated in version 3.0.0: The handler is also called when a client reconnects.*
+        """
         self.disconnect_handlers.append(handler)
 
-    def handle_handshake(self, socket_id: str, document_id: str, next_message_id: Optional[int]) -> None:
+    def on_delete(self, handler: Callable[..., Any] | Awaitable) -> None:
+        """Add a callback to be invoked when the client is deleted.
+
+        The callback has an optional parameter of `nicegui.Client`.
+
+        *Added in version 3.0.0*
+        """
+        self.delete_handlers.append(handler)
+
+    def handle_handshake(self, socket_id: str, document_id: str, next_message_id: int | None) -> None:
         """Cancel pending disconnect task and invoke connect handlers."""
+        self._waiting_for_connection.clear()
+        self._connected.set()
         self._socket_to_document_id[socket_id] = document_id
         self._cancel_delete_task(document_id)
         self._num_connections[document_id] += 1
@@ -262,29 +295,25 @@ class Client:
             self.safe_invoke(t)
 
     def handle_disconnect(self, socket_id: str) -> None:
-        """Wait for the browser to reconnect; invoke disconnect handlers if it doesn't.
-
-        NOTE:
-        In contrast to connect handlers, disconnect handlers are not called during a reconnect.
-        This behavior should be fixed in version 3.0.
-        """
+        """Wait for the browser to reconnect; invoke deletion handlers if it doesn't."""
         if socket_id not in self._socket_to_document_id:
             return
         document_id = self._socket_to_document_id.pop(socket_id)
         self._cancel_delete_task(document_id)
         self._num_connections[document_id] -= 1
+        self.tab_id = None
+
+        for t in self.disconnect_handlers:
+            self.safe_invoke(t)
+        for t in core.app._disconnect_handlers:  # pylint: disable=protected-access
+            self.safe_invoke(t)
 
         async def delete_content() -> None:
             await asyncio.sleep(self.page.resolve_reconnect_timeout())
             if self._num_connections[document_id] == 0:
-                for t in self.disconnect_handlers:
-                    self.safe_invoke(t)
-                for t in core.app._disconnect_handlers:  # pylint: disable=protected-access
-                    self.safe_invoke(t)
                 self._num_connections.pop(document_id)
                 self._delete_tasks.pop(document_id)
-                if not self.shared:
-                    self.delete()
+                self.delete()
         self._delete_tasks[document_id] = \
             background_tasks.create(delete_content(), name=f'delete content {document_id}')
 
@@ -292,7 +321,7 @@ class Client:
         if document_id in self._delete_tasks:
             self._delete_tasks.pop(document_id).cancel()
 
-    def handle_event(self, msg: Dict) -> None:
+    def handle_event(self, msg: dict) -> None:
         """Forward an event to the corresponding element."""
         with self:
             sender = self.elements.get(msg['id'])
@@ -302,11 +331,11 @@ class Client:
                     msg['args'] = msg['args'][0]
                 sender._handle_event(msg)  # pylint: disable=protected-access
 
-    def handle_javascript_response(self, msg: Dict) -> None:
+    def handle_javascript_response(self, msg: dict) -> None:
         """Store the result of a JavaScript command."""
         JavaScriptRequest.resolve(msg['request_id'], msg.get('result'))
 
-    def safe_invoke(self, func: Union[Callable[..., Any], Awaitable]) -> None:
+    def safe_invoke(self, func: Callable[..., Any] | Awaitable) -> None:
         """Invoke the potentially async function in the client context and catch any exceptions."""
         func_name = func.__name__ if hasattr(func, '__name__') else str(func)
         try:
@@ -346,9 +375,16 @@ class Client:
         If the global clients dictionary does not contain the client, its elements are still removed and a KeyError is raised.
         Normally this should never happen, but has been observed (see #1826).
         """
+        for t in self.delete_handlers:
+            self.safe_invoke(t)
+        for t in core.app._delete_handlers:  # pylint: disable=protected-access
+            self.safe_invoke(t)
+        self._waiting_for_disconnect.clear()
+        self._deleted_event.set()
         self.remove_all_elements()
         self.outbox.stop()
-        del Client.instances[self.id]
+        if self.id in Client.instances:
+            del Client.instances[self.id]
         self._deleted = True
 
     def check_existence(self) -> None:
@@ -359,32 +395,18 @@ class Client:
                               'See https://github.com/zauberzeug/nicegui/issues/3028 for more information.',
                               stack_info=True)
 
-    @contextmanager
-    def individual_target(self, socket_id: str) -> Iterator[None]:
-        """Use individual socket ID while in this context.
-
-        This context is useful for limiting messages from the shared auto-index page to a single client.
-        """
-        self._temporary_socket_id = socket_id
-        yield
-        self._temporary_socket_id = None
-
     @classmethod
-    async def prune_instances(cls) -> None:
-        """Prune stale clients in an endless loop."""
-        while True:
-            try:
-                stale_clients = [
-                    client
-                    for client in cls.instances.values()
-                    if not client.shared and not client.has_socket_connection and client.created < time.time() - 60.0
-                ]
-                for client in stale_clients:
-                    client.delete()
-            except Exception:
-                # NOTE: make sure the loop doesn't crash
-                log.exception('Error while pruning clients')
-            try:
-                await asyncio.sleep(10)
-            except asyncio.CancelledError:
-                break
+    def prune_instances(cls, *, client_age_threshold: float = 60.0) -> None:
+        """Prune stale clients."""
+        try:
+            stale_clients = [
+                client
+                for client in cls.instances.values()
+                if not client.has_socket_connection and client.created <= time.time() - client_age_threshold
+            ]
+            for client in stale_clients:
+                log.debug(f'Pruning stale client {client.id}')
+                client.delete()
+
+        except Exception:
+            log.exception('Error while pruning clients')
