@@ -1,14 +1,14 @@
 from __future__ import annotations
 
 import functools
+from collections.abc import Iterable
 from dataclasses import dataclass
 from pathlib import Path
-from typing import TYPE_CHECKING, Callable, Dict, Iterable, List, Optional, Set, Tuple
-
-import vbuild
+from typing import TYPE_CHECKING, Callable
 
 from .dataclasses import KWONLY_SLOTS
 from .helpers import hash_file_path
+from .vbuild import VBuild
 from .version import __version__
 
 if TYPE_CHECKING:
@@ -56,17 +56,23 @@ class Library:
     key: str
     name: str
     path: Path
-    expose: bool
 
 
-vue_components: Dict[str, VueComponent] = {}
-js_components: Dict[str, JsComponent] = {}
-libraries: Dict[str, Library] = {}
-resources: Dict[str, Resource] = {}
-dynamic_resources: Dict[str, DynamicResource] = {}
+@dataclass(**KWONLY_SLOTS)
+class EsmModule:
+    name: str
+    path: Path
 
 
-def register_vue_component(path: Path, *, max_time: Optional[float]) -> Component:
+vue_components: dict[str, VueComponent] = {}
+js_components: dict[str, JsComponent] = {}
+libraries: dict[str, Library] = {}
+resources: dict[str, Resource] = {}
+dynamic_resources: dict[str, DynamicResource] = {}
+esm_modules: dict[str, EsmModule] = {}
+
+
+def register_vue_component(path: Path, *, max_time: float | None) -> Component:
     """Register a .vue or .js Vue component.
 
     Single-file components (.vue) are built right away
@@ -79,7 +85,7 @@ def register_vue_component(path: Path, *, max_time: Optional[float]) -> Componen
         if key in vue_components and vue_components[key].path == path:
             return vue_components[key]
         assert key not in vue_components, f'Duplicate VUE component {key}'
-        v = vbuild.VBuild(path.name, path.read_text(encoding='utf-8'))
+        v = VBuild(path)
         vue_components[key] = VueComponent(key=key, name=name, path=path, html=v.html, script=v.script, style=v.style)
         return vue_components[key]
     if path.suffix == '.js':
@@ -91,7 +97,7 @@ def register_vue_component(path: Path, *, max_time: Optional[float]) -> Componen
     raise ValueError(f'Unsupported component type "{path.suffix}"')
 
 
-def register_library(path: Path, *, expose: bool = False, max_time: Optional[float]) -> Library:
+def register_library(path: Path, *, max_time: float | None) -> Library:
     """Register a *.js library."""
     key = compute_key(path, max_time=max_time)
     name = _get_name(path)
@@ -99,12 +105,12 @@ def register_library(path: Path, *, expose: bool = False, max_time: Optional[flo
         if key in libraries and libraries[key].path == path:
             return libraries[key]
         assert key not in libraries, f'Duplicate js library {key}'
-        libraries[key] = Library(key=key, name=name, path=path, expose=expose)
+        libraries[key] = Library(key=key, name=name, path=path)
         return libraries[key]
     raise ValueError(f'Unsupported library type "{path.suffix}"')
 
 
-def register_resource(path: Path, *, max_time: Optional[float]) -> Resource:
+def register_resource(path: Path, *, max_time: float | None) -> Resource:
     """Register a resource."""
     key = compute_key(path, max_time=max_time)
     if key in resources and resources[key].path == path:
@@ -120,8 +126,15 @@ def register_dynamic_resource(name: str, function: Callable) -> DynamicResource:
     return dynamic_resources[name]
 
 
-@functools.lru_cache(maxsize=None)
-def compute_key(path: Path, *, max_time: Optional[float]) -> str:
+def register_esm(name: str, path: Path, *, max_time: float | None) -> None:
+    """Register an ESM module."""
+    if any(name == esm_module.name for esm_module in esm_modules.values()):
+        raise ValueError(f'Duplicate ESM module name "{name}"')
+    esm_modules[compute_key(path, max_time=max_time)] = EsmModule(name=name, path=path)
+
+
+@functools.cache
+def compute_key(path: Path, *, max_time: float | None) -> str:
     """Compute a key for a given path using a hash function.
 
     If the path is relative to the NiceGUI base directory, the key is computed from the relative path.
@@ -140,28 +153,33 @@ def _get_name(path: Path) -> str:
     return path.name.split('.', 1)[0]
 
 
-def generate_resources(prefix: str, elements: Iterable[Element]) -> Tuple[List[str],
-                                                                          List[str],
-                                                                          List[str],
-                                                                          Dict[str, str],
-                                                                          List[str],
-                                                                          List[str]]:
+def generate_resources(prefix: str, elements: Iterable[Element]) -> tuple[list[str],
+                                                                          list[str],
+                                                                          list[str],
+                                                                          dict[str, str],
+                                                                          list[str],
+                                                                          list[str]]:
     """Generate the resources required by the elements to be sent to the client."""
-    done_libraries: Set[str] = set()
-    done_components: Set[str] = set()
-    vue_scripts: List[str] = []
-    vue_html: List[str] = []
-    vue_styles: List[str] = []
-    imports: Dict[str, str] = {}
-    js_imports: List[str] = []
-    js_imports_urls: List[str] = []
+    done_libraries: set[str] = set()
+    done_components: set[str] = set()
+    vue_scripts: list[str] = []
+    vue_html: list[str] = []
+    vue_styles: list[str] = []
+    imports: dict[str, str] = {}
+    js_imports: list[str] = []
+    js_imports_urls: list[str] = []
 
-    # build the importmap structure for exposed libraries
+    # build the importmap structure for libraries
     for key, library in libraries.items():
-        if key not in done_libraries and library.expose:
+        if key not in done_libraries:
             imports[library.name] = f'{prefix}/_nicegui/{__version__}/libraries/{key}'
             done_libraries.add(key)
 
+    # build the importmap structure for ESM modules
+    for key, esm_module in esm_modules.items():
+        imports[f'{esm_module.name}'] = f'{prefix}/_nicegui/{__version__}/esm/{key}/index.js'
+        imports[f'{esm_module.name}/'] = f'{prefix}/_nicegui/{__version__}/esm/{key}/'
+
     # build the none-optimized component (i.e. the Vue component)
     for key, vue_component in vue_components.items():
         if key not in done_components:
@@ -173,13 +191,6 @@ def generate_resources(prefix: str, elements: Iterable[Element]) -> Tuple[List[s
 
     # build the resources associated with the elements
     for element in elements:
-        for library in element.libraries:
-            if library.key not in done_libraries:
-                if not library.expose:
-                    url = f'{prefix}/_nicegui/{__version__}/libraries/{library.key}'
-                    js_imports.append(f'import "{url}";')
-                    js_imports_urls.append(url)
-                done_libraries.add(library.key)
         if element.component:
             js_component = element.component
             if js_component.key not in done_components and js_component.path.suffix.lower() == '.js':
