@@ -1,10 +1,10 @@
 from typing import Optional
 
-from ..colors import set_text_color
-from ..element import Element
+from .mixins.color_elements import TextColorElement
+from .mixins.name_element import NameElement
 
 
-class Icon(Element):
+class Icon(NameElement, TextColorElement):
 
     def __init__(self,
                  name: str,
@@ -16,16 +16,13 @@ class Icon(Element):
 
         This element is based on Quasar's `QIcon <https://quasar.dev/vue-components/icon>`_ component.
 
-        `Here <https://material.io/icons/>`_ is a reference of possible names.
+        `Here <https://fonts.google.com/icons?icon.set=Material+Icons>`_ is a reference of possible names.
 
-        :param name: name of the icon
+        :param name: name of the icon (snake case, e.g. `add_circle`)
         :param size: size in CSS units, including unit name or standard size name (xs|sm|md|lg|xl), examples: 16px, 2rem
         :param color: icon color (either a Quasar, Tailwind, or CSS color or `None`, default: `None`)
         """
-        super().__init__('q-icon')
-        self._props['name'] = name
+        super().__init__(tag='q-icon', name=name, text_color=color)
 
         if size:
             self._props['size'] = size
-
-        set_text_color(self, color)
