@@ -1,23 +1,26 @@
 from typing import Optional
 
-from ..colors import set_text_color
+from ..events import Handler, ValueChangeEventArguments
 from .label import Label
+from .mixins.color_elements import TextColorElement
+from .mixins.disableable_element import DisableableElement
 from .mixins.value_element import ValueElement
 
 
-class Knob(ValueElement):
+class Knob(ValueElement, DisableableElement, TextColorElement):
 
     def __init__(self,
                  value: float = 0.0,
                  *,
-                 min: float = 0.0,
-                 max: float = 1.0,
+                 min: float = 0.0,  # pylint: disable=redefined-builtin
+                 max: float = 1.0,  # pylint: disable=redefined-builtin
                  step: float = 0.01,
                  color: Optional[str] = 'primary',
                  center_color: Optional[str] = None,
                  track_color: Optional[str] = None,
                  size: Optional[str] = None,
                  show_value: bool = False,
+                 on_change: Optional[Handler[ValueChangeEventArguments]] = None,
                  ) -> None:
         """Knob
 
@@ -33,13 +36,13 @@ class Knob(ValueElement):
         :param track_color: color name for the track of the component, examples: primary, teal-10
         :param size: size in CSS units, including unit name or standard size name (xs|sm|md|lg|xl), examples: 16px, 2rem
         :param show_value: whether to show the value as text
+        :param on_change: callback to execute when the value changes
         """
-        super().__init__(tag='q-knob', value=value, on_value_change=None, throttle=0.05)
+        super().__init__(tag='q-knob', value=value, on_value_change=on_change, throttle=0.05, text_color=color)
 
         self._props['min'] = min
         self._props['max'] = max
         self._props['step'] = step
-        set_text_color(self, color)
         self._props['show-value'] = True  # NOTE: enable default slot, e.g. for nested icon
 
         if center_color:
