@@ -18,7 +18,8 @@ class TextElement(Element):
     def bind_text_to(self,
                      target_object: Any,
                      target_name: str = 'text',
-                     forward: Optional[Callable[[Any], Any]] = None,
+                     forward: Optional[Callable[[Any], Any]] = None, *,
+                     strict: Optional[bool] = None,
                      ) -> Self:
         """Bind the text of this element to the target object's target_name property.
 
@@ -28,14 +29,17 @@ class TextElement(Element):
         :param target_object: The object to bind to.
         :param target_name: The name of the property to bind to.
         :param forward: A function to apply to the value before applying it to the target (default: identity).
+        :param strict: Whether to check (and raise) if the target object has the specified property (default: None,
+            performs a check if the object is not a dictionary, *added in version 3.0.0*).
         """
-        bind_to(self, 'text', target_object, target_name, forward)
+        bind_to(self, 'text', target_object, target_name, forward, self_strict=False, other_strict=strict)
         return self
 
     def bind_text_from(self,
                        target_object: Any,
                        target_name: str = 'text',
-                       backward: Optional[Callable[[Any], Any]] = None,
+                       backward: Optional[Callable[[Any], Any]] = None, *,
+                       strict: Optional[bool] = None,
                        ) -> Self:
         """Bind the text of this element from the target object's target_name property.
 
@@ -45,8 +49,10 @@ class TextElement(Element):
         :param target_object: The object to bind from.
         :param target_name: The name of the property to bind from.
         :param backward: A function to apply to the value before applying it to this element (default: identity).
+        :param strict: Whether to check (and raise) if the target object has the specified property (default: None,
+            performs a check if the object is not a dictionary, *added in version 3.0.0*).
         """
-        bind_from(self, 'text', target_object, target_name, backward)
+        bind_from(self, 'text', target_object, target_name, backward, self_strict=False, other_strict=strict)
         return self
 
     def bind_text(self,
@@ -54,6 +60,7 @@ class TextElement(Element):
                   target_name: str = 'text', *,
                   forward: Optional[Callable[[Any], Any]] = None,
                   backward: Optional[Callable[[Any], Any]] = None,
+                  strict: Optional[bool] = None,
                   ) -> Self:
         """Bind the text of this element to the target object's target_name property.
 
@@ -65,8 +72,12 @@ class TextElement(Element):
         :param target_name: The name of the property to bind to.
         :param forward: A function to apply to the value before applying it to the target (default: identity).
         :param backward: A function to apply to the value before applying it to this element (default: identity).
+        :param strict: Whether to check (and raise) if the target object has the specified property (default: None,
+            performs a check if the object is not a dictionary, *added in version 3.0.0*).
         """
-        bind(self, 'text', target_object, target_name, forward=forward, backward=backward)
+        bind(self, 'text', target_object, target_name,
+             forward=forward, backward=backward,
+             self_strict=False, other_strict=strict)
         return self
 
     def set_text(self, text: str) -> None:
