@@ -1,4 +1,4 @@
-from typing import Any, Dict, List, Optional, Union
+from typing import Any, Optional, Union
 
 from ..events import GenericEventArguments, Handler, ValueChangeEventArguments
 from .choice_element import ChoiceElement
@@ -8,7 +8,7 @@ from .mixins.disableable_element import DisableableElement
 class Radio(ChoiceElement, DisableableElement):
 
     def __init__(self,
-                 options: Union[List, Dict], *,
+                 options: Union[list, dict], *,
                  value: Any = None,
                  on_change: Optional[Handler[ValueChangeEventArguments]] = None,
                  ) -> None:
