@@ -1,18 +1,23 @@
-from typing import Callable, Optional
+from typing import Optional
 
+from ..events import Handler, ValueChangeEventArguments
+from .mixins.disableable_element import DisableableElement
 from .mixins.value_element import ValueElement
 
 
-class Slider(ValueElement):
+class Slider(ValueElement, DisableableElement):
 
     def __init__(self, *,
-                 min: float,
-                 max: float,
+                 min: float,  # pylint: disable=redefined-builtin
+                 max: float,  # pylint: disable=redefined-builtin
                  step: float = 1.0,
                  value: Optional[float] = None,
-                 on_change: Optional[Callable] = None) -> None:
+                 on_change: Optional[Handler[ValueChangeEventArguments]] = None,
+                 ) -> None:
         """Slider
 
+        This element is based on Quasar's `QSlider <https://quasar.dev/vue-components/slider>`_ component.
+
         :param min: lower bound of the slider
         :param max: upper bound of the slider
         :param step: step size
