@@ -1,18 +1,20 @@
-from typing import Callable, Optional
+from typing import Optional
 
+from ..events import Handler, ValueChangeEventArguments
+from .mixins.disableable_element import DisableableElement
 from .mixins.value_element import ValueElement
 
 
-class Time(ValueElement):
+class Time(ValueElement, DisableableElement):
 
     def __init__(self,
-                 value: Optional[str] = None,
-                 *,
+                 value: Optional[str] = None, *,
                  mask: str = 'HH:mm',
-                 on_change: Optional[Callable] = None) -> None:
+                 on_change: Optional[Handler[ValueChangeEventArguments]] = None,
+                 ) -> None:
         """Time Input
 
-        This element is based on Quasar's `QTime <https://quasar.dev/vue-components/date>`_ component.
+        This element is based on Quasar's `QTime <https://quasar.dev/vue-components/time>`_ component.
         The time is a string in the format defined by the `mask` parameter.
 
         :param value: the initial time
