@@ -1,4 +1,5 @@
-from typing import Any, Dict, Iterator, List, Literal, Optional, Set
+from collections.abc import Iterator
+from typing import Any, Literal, Optional
 
 from typing_extensions import Self
 
@@ -9,7 +10,7 @@ from .mixins.filter_element import FilterElement
 class Tree(FilterElement):
 
     def __init__(self,
-                 nodes: List[Dict], *,
+                 nodes: list[dict], *,
                  node_key: str = 'id',
                  label_key: str = 'label',
                  children_key: str = 'children',
@@ -59,24 +60,32 @@ class Tree(FilterElement):
         def update_prop(name: str, value: Any) -> None:
             if self._props[name] != value:
                 self._props[name] = value
-                self.update()
 
         def handle_selected(e: GenericEventArguments) -> None:
+            previous_value = self._props.get('selected')
             update_prop('selected', e.args)
+            args = ValueChangeEventArguments(sender=self, client=self.client,
+                                             value=e.args, previous_value=previous_value)
             for handler in self._select_handlers:
-                handle_event(handler, ValueChangeEventArguments(sender=self, client=self.client, value=e.args))
+                handle_event(handler, args)
         self.on('update:selected', handle_selected)
 
         def handle_expanded(e: GenericEventArguments) -> None:
+            previous_value = self._props.get('expanded')
             update_prop('expanded', e.args)
+            args = ValueChangeEventArguments(sender=self, client=self.client,
+                                             value=e.args, previous_value=previous_value)
             for handler in self._expand_handlers:
-                handle_event(handler, ValueChangeEventArguments(sender=self, client=self.client, value=e.args))
+                handle_event(handler, args)
         self.on('update:expanded', handle_expanded)
 
         def handle_ticked(e: GenericEventArguments) -> None:
+            previous_value = self._props.get('ticked')
             update_prop('ticked', e.args)
+            args = ValueChangeEventArguments(sender=self, client=self.client,
+                                             value=e.args, previous_value=previous_value)
             for handler in self._tick_handlers:
-                handle_event(handler, ValueChangeEventArguments(sender=self, client=self.client, value=e.args))
+                handle_event(handler, args)
         self.on('update:ticked', handle_ticked)
 
     def on_select(self, callback: Handler[ValueChangeEventArguments]) -> Self:
@@ -93,7 +102,6 @@ class Tree(FilterElement):
         self._props.setdefault('selected', None)
         if self._props['selected'] != node_key:
             self._props['selected'] = node_key
-            self.update()
         return self
 
     def deselect(self) -> Self:
@@ -113,55 +121,59 @@ class Tree(FilterElement):
         self._tick_handlers.append(callback)
         return self
 
-    def tick(self, node_keys: Optional[List[str]] = None) -> Self:
+    def tick(self, node_keys: Optional[list[str]] = None) -> Self:
         """Tick the given nodes.
 
         :param node_keys: list of node keys to tick or ``None`` to tick all nodes (default: ``None``)
         """
         self._props.setdefault('ticked', [])
         self._props['ticked'][:] = self._find_node_keys(node_keys).union(self._props['ticked'])
-        self.update()
         return self
 
-    def untick(self, node_keys: Optional[List[str]] = None) -> Self:
+    def untick(self, node_keys: Optional[list[str]] = None) -> Self:
         """Remove tick from the given nodes.
 
         :param node_keys: list of node keys to untick or ``None`` to untick all nodes (default: ``None``)
         """
         self._props.setdefault('ticked', [])
         self._props['ticked'][:] = set(self._props['ticked']).difference(self._find_node_keys(node_keys))
-        self.update()
         return self
 
-    def expand(self, node_keys: Optional[List[str]] = None) -> Self:
+    def expand(self, node_keys: Optional[list[str]] = None) -> Self:
         """Expand the given nodes.
 
         :param node_keys: list of node keys to expand (default: all nodes)
         """
         self._props.setdefault('expanded', [])
         self._props['expanded'][:] = self._find_node_keys(node_keys).union(self._props['expanded'])
-        self.update()
         return self
 
-    def collapse(self, node_keys: Optional[List[str]] = None) -> Self:
+    def collapse(self, node_keys: Optional[list[str]] = None) -> Self:
         """Collapse the given nodes.
 
         :param node_keys: list of node keys to collapse (default: all nodes)
         """
         self._props.setdefault('expanded', [])
         self._props['expanded'][:] = set(self._props['expanded']).difference(self._find_node_keys(node_keys))
-        self.update()
         return self
 
-    def _find_node_keys(self, node_keys: Optional[List[str]] = None) -> Set[str]:
-        if node_keys is not None:
-            return set(node_keys)
+    def nodes(self, *, visible: Optional[bool] = None) -> Iterator[dict]:
+        """Iterate over all nodes.
 
-        CHILDREN_KEY = self._props['children-key']
-        NODE_KEY = self._props['node-key']
-
-        def iterate_nodes(nodes: List[Dict]) -> Iterator[Dict]:
+        :param visible: if ``True``, only visible nodes are returned; if ``False``, only invisible nodes are returned; if ``None``, all nodes are returned (default: ``None``)
+        """
+        def iterate_nodes(nodes: list[dict]) -> Iterator[dict]:
+            expanded = self._props.get('expanded')
+            NODE_KEY = self._props['node-key']
+            CHILDREN_KEY = self._props['children-key']
             for node in nodes:
                 yield node
-                yield from iterate_nodes(node.get(CHILDREN_KEY, []))
-        return {node[NODE_KEY] for node in iterate_nodes(self._props['nodes'])}
+                is_expanded = expanded is None or node[NODE_KEY] in expanded
+                if (is_expanded and visible is not False) or (not is_expanded and visible is not True):
+                    yield from iterate_nodes(node.get(CHILDREN_KEY, []))
+        return iterate_nodes(self._props['nodes'])
+
+    def _find_node_keys(self, node_keys: Optional[list[str]] = None) -> set[str]:
+        if node_keys is not None:
+            return set(node_keys)
+        return {node[self._props['node-key']] for node in self.nodes()}
