@@ -1,23 +1,23 @@
-import warnings
+from pathlib import Path
+from typing import Union
 
-from ..dependencies import register_component
-from ..element import Element
-
-register_component('video', __file__, 'video.js')
+from .mixins.source_element import SourceElement
 
 
-class Video(Element):
+class Video(SourceElement, component='video.js'):
+    SOURCE_IS_MEDIA_FILE = True
 
-    def __init__(self, src: str, *,
+    def __init__(self, src: Union[str, Path], *,
                  controls: bool = True,
                  autoplay: bool = False,
                  muted: bool = False,
                  loop: bool = False,
-                 type: str = '',  # DEPRECATED
                  ) -> None:
         """Video
 
-        :param src: URL of the video source
+        Displays a video.
+
+        :param src: URL or local file path of the video source
         :param controls: whether to show the video controls, like play, pause, and volume (default: `True`)
         :param autoplay: whether to start playing the video automatically (default: `False`)
         :param muted: whether the video should be initially muted (default: `False`)
@@ -26,13 +26,26 @@ class Video(Element):
         See `here <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/video#events>`_
         for a list of events you can subscribe to using the generic event subscription `on()`.
         """
-        super().__init__('video')
-        self._props['src'] = src
+        super().__init__(source=src)
         self._props['controls'] = controls
         self._props['autoplay'] = autoplay
         self._props['muted'] = muted
         self._props['loop'] = loop
 
-        if type:
-            url = f'https://github.com/zauberzeug/nicegui/pull/624'
-            warnings.warn(DeprecationWarning(f'The type parameter for ui.video is deprecated and ineffective ({url}).'))
+    def set_source(self, source: Union[str, Path]) -> None:
+        return super().set_source(source)
+
+    def seek(self, seconds: float) -> None:
+        """Seek to a specific position in the video.
+
+        :param seconds: the position in seconds
+        """
+        self.run_method('seek', seconds)
+
+    def play(self) -> None:
+        """Play video."""
+        self.run_method('play')
+
+    def pause(self) -> None:
+        """Pause video."""
+        self.run_method('pause')
