@@ -29,7 +29,7 @@ def error_content(status_code: int, exception: Union[str, Exception] = '') -> No
             message += ': ' + str(exception)
 
     with column().style('width: 100%; padding: 5rem 0; align-items: center; gap: 0'):
-        html(SAD_FACE_SVG).style('width: 8rem; padding: 1.25rem 0')
+        html(SAD_FACE_SVG, sanitize=False).style('width: 8rem; padding: 1.25rem 0')
         label(str(status_code)).style('font-size: 3.75rem; line-height: 1; padding: 1.25rem 0')
         label(title).style('font-size: 1.25rem; line-height: 1.75rem; padding: 1.25rem 0')
         label(message).style('font-size: 1.125rem; line-height: 1.75rem; color: rgb(107 114 128)')
