@@ -1,11 +1,11 @@
 from __future__ import annotations
 
 import asyncio
-from typing import Any, ClassVar, Dict
+from typing import Any, ClassVar
 
 
 class JavaScriptRequest:
-    _instances: ClassVar[Dict[str, JavaScriptRequest]] = {}
+    _instances: ClassVar[dict[str, JavaScriptRequest]] = {}
 
     def __init__(self, request_id: str, *, timeout: float) -> None:
         self.request_id = request_id
