@@ -129,6 +129,7 @@ __all__ = [
     'update',
     'upload',
     'video',
+    'xterm',
 ]
 
 from .context import context
@@ -211,7 +212,7 @@ from .elements.rating import Rating as rating
 from .elements.restructured_text import ReStructuredText as restructured_text
 from .elements.row import Row as row
 from .elements.scene import Scene as scene
-from .elements.scene_view import SceneView as scene_view
+from .elements.scene import SceneView as scene_view
 from .elements.scroll_area import ScrollArea as scroll_area
 from .elements.select import Select as select
 from .elements.separator import Separator as separator
@@ -242,6 +243,7 @@ from .elements.tooltip import Tooltip as tooltip
 from .elements.tree import Tree as tree
 from .elements.upload import Upload as upload
 from .elements.video import Video as video
+from .elements.xterm import Xterm as xterm
 from .functions import clipboard
 from .functions.download import download
 from .functions.html import add_body_html, add_head_html
